<?php

declare(strict_types=1);

namespace Z14\Z14ComvahroBewerber3\Tests\Unit\Controller;

use PHPUnit\Framework\MockObject\MockObject;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
use TYPO3\TestingFramework\Core\AccessibleObjectInterface;
use TYPO3\TestingFramework\Core\Unit\UnitTestCase;

/**
 * Test case
 */
class RenderControllerTest extends UnitTestCase
{
    /**
     * @var \Z14\Z14ComvahroBewerber3\Controller\RenderController|MockObject|AccessibleObjectInterface
     */
    protected $subject;

    protected function setUp(): void
    {
        parent::setUp();
        $this->subject = $this->getMockBuilder($this->buildAccessibleProxy(\Z14\Z14ComvahroBewerber3\Controller\RenderController::class))
            ->onlyMethods(['redirect', 'forward', 'addFlashMessage'])
            ->disableOriginalConstructor()
            ->getMock();
    }

    protected function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function listActionFetchesAllRendersFromRepositoryAndAssignsThemToView(): void
    {
        $allRenders = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->disableOriginalConstructor()
            ->getMock();

        $renderRepository = $this->getMockBuilder(\Z14\Z14ComvahroBewerber3\Domain\Repository\RenderRepository::class)
            ->onlyMethods(['findAll'])
            ->disableOriginalConstructor()
            ->getMock();
        $renderRepository->expects(self::once())->method('findAll')->will(self::returnValue($allRenders));
        $this->subject->_set('renderRepository', $renderRepository);

        $view = $this->getMockBuilder(ViewInterface::class)->getMock();
        $view->expects(self::once())->method('assign')->with('renders', $allRenders);
        $this->subject->_set('view', $view);

        $this->subject->listAction();
    }

    /**
     * @test
     */
    public function showActionAssignsTheGivenRenderToView(): void
    {
        $render = new \Z14\Z14ComvahroBewerber3\Domain\Model\Render();

        $view = $this->getMockBuilder(ViewInterface::class)->getMock();
        $this->subject->_set('view', $view);
        $view->expects(self::once())->method('assign')->with('render', $render);

        $this->subject->showAction($render);
    }
}
